<?php
// $Id: menu.inc,v 1.1 2008/02/17 05:22:40 sun Exp $

/**
 * @file
 * Translation service for Drupal 5.x menu system.
 */

/**
 * Translates the menu tree and caches the result.
 */
function translatable_translate_menu() {
  global $_menu, $user;

  // @todo cache_get() ?!

  foreach ($_menu['items'] as $mid => $item) {
    if ($item['type'] & (MENU_CREATED_BY_ADMIN | MENU_MODIFIED_BY_ADMIN)) {
      $expanded = false;
      
      $path = translatable_get_localizedpath($item['path'], translatable_get_defaultcontentlocale());
      $_menu['items'][$mid]['path'] = $path;
      $reqpath = $_GET['q'];
      
      $drupalpath = drupal_get_normal_path($reqpath);
      if ($drupalpath == $path) {
        $expanded = true;
      }
      
      if (!$expanded) {
        $drupalpath = drupal_get_normal_path(translatable_get_path($reqpath));
        if ($drupalpath == $path) {
          $expanded = true;
        }
      }
      
      if (!$expanded) {
        $languages = translatable_available_locales();
        foreach ($languages as $langprefix => $langname) {
          if (!$expanded) {
            $drupalpath = drupal_get_normal_path(translatable_get_path($reqpath, $langprefix));
            if ($drupalpath == $path) {
              $expanded = true;
              break;
            }
          }
        }
      }
      
      $translation = translatable_translate_menu_item($mid);
      $_menu['items'][$mid]['title'] = !isset($translation['title']) ? $item['title'] : $translation['title'];
      $_menu['items'][$mid]['description'] = !isset($translation['description']) ? $item['description'] : $translation['description'];
      
      if ($expanded) {
        $_menu['items'][$mid]['path'] = drupal_get_normal_path($reqpath);
        $locations = translatable_get_breadcrumb($mid);
        $locpath = translatable_get_localizedpath(variable_get('site_frontpage', 'node'), translatable_get_defaultcontentlocale());
        $locations[$locpath] = l(t('Home'), $locpath);
        $locations = array_reverse($locations);
        array_pop($locations);
        drupal_set_breadcrumb($locations);
      }
    }
  }

  // Update cache
  $cid = "$user->uid:" . translatable_get_locale();
  cache_set($cid, 'cache_menu', serialize($_menu), time() + (60 * 60 * 24));
}

/**
 * Fetches the translation for a single menu item.
 * To reduce database load, all translations for the current locale are cached.
 *
 * @param $mid
 *   The menu item id to translate.
 */
function translatable_translate_menu_item($mid) {
  static $translations = array();

  if (!isset($translations[$mid])) {
    $items = translatable_find('translation', "object_name = 'menu' AND locale = '". translatable_get_locale() ."'");
    foreach ($items as $key => $item) {
      $translations[$item['object_key']][$item['object_field']] = $item['translation'];
    }
  }
  return $translations[$mid];
}

/**
 * Builds the breadcrumb path for a menu item.
 *
 * @param $mid
 *   The menu item id to build the breadcrumb path for.
 */
function translatable_get_breadcrumb($mid) {
  global $_menu;
  static $locations = array();

  if ($mid > 0) {
    $_menu['visible'][$mid]['type'] |= MENU_EXPANDED;
    if ($_menu['visible'][$mid]['type'] & MENU_VISIBLE_IN_BREADCRUMB) {
      $locpath = translatable_get_localizedpath($_menu['visible'][$mid]['path'], translatable_get_defaultcontentlocale());
      $locations[$locpath] = l($_menu['items'][$mid]['title'], $locpath);
    }
    translatable_get_breadcrumb($_menu['visible'][$mid]['pid']);
  }
  return $locations;
}

