<?php
// $Id: translatable.block.inc,v 1.1 2008/02/17 05:22:40 sun Exp $

/**
 * @file
 * Provides language and translation language blocks and user integration.
 */

/**
 * Implementation of hook_block().
 */
function translatable_block($op = 'list', $delta = 0, $edit = array()) {
  switch ($op) {
    case 'list':
      $block['ui']['info']   = t('Language');
      $block['ui']['region'] = 'right';
      $block['ui']['status'] = 1;
      $block['translation']['info']   = t('Translation language');
      $block['translation']['region'] = 'right';
      $block['translation']['status'] = 1;
      $block['translation']['weight'] = -10;
      break;
    
    case 'configure':
      $form = array();
      if ($delta == 'ui' || $delta == 'translation') {
        $form['translatable_block_'. $delta .'_hidecurrentlocale'] = array(
          '#type' => 'checkbox',
          '#title' => t('Hide current language'),
          '#description' => t('If this option is enabled, the current language is not displayed in the language switching block.'),
          '#default_value' => variable_get('translatable_block_'. $delta .'_hidecurrentlocale', TRUE),
        );
        $flag_options = array(
          'flag' => t('Show flags'),
          'name' => t('Show language names'),
          'flag-name' => t('Show flags and language names'),
        );
        $form['translatable_block_'. $delta .'_style'] = array(
          '#type' => 'select',
          '#title' => t('Display style'),
          '#options' => $flag_options,
          '#default_value' => variable_get('translatable_block_'. $delta .'_style', 'flag-name'),
        );
        $form['translatable_block_'. $delta .'_flagspath'] = array(
          '#type' => 'textfield',
          '#title' => t('Flag icons path'),
          '#default_value' => variable_get('translatable_block_'. $delta .'_flagspath', drupal_get_path('module', 'translatable') .'/flags/*.png'),
          '#size' => 50,
          '#maxlength' => 180,
          '#description' => t('Path for flag icons, relative to the Drupal installation. \'*\' is a placeholder for the ISO language code.'),
        );
      }
      return $form;
      
    case 'save':
      if ($delta == 'ui' || $delta == 'translation') {
        variable_set('translatable_block_'. $delta .'_hidecurrentlocale', $edit['translatable_block_'. $delta .'_hidecurrentlocale']);
        variable_set('translatable_block_'. $delta .'_style', $edit['translatable_block_'. $delta .'_style']);
        variable_set('translatable_block_'. $delta .'_flagspath', $edit['translatable_block_'. $delta .'_flagspath']);
      }
      return;
      
    case 'view':
      if ($delta == 'ui') {
        $block['subject'] = t('Language');
        $block['content'] = theme('item_list', theme('translatable_languages', 'ui'));
      }
      else if ($delta == 'translation') {
        $block['subject'] = t('Translation language');
        $languages = translatable_adminlocale_formselect(NULL, NULL);
        if (!empty($languages)) {
          $block['content'] = theme('item_list', $languages);
        }
      }
      break;
  }
  return $block;
}

/**
 * Create the links for the switching language block
 *
 * @param string $block
 *   The Translatable block name, can be 'ui', 'content' or 'translation'.
 * @param array $form
 *   The form on which the languages should be based on.
 *
 * @return
 *   A HTML list of links of available languages.
 */
function theme_translatable_languages($block = 'ui', $form = NULL) {
  global $user;
  static $tset, $nodes;

  drupal_add_css(drupal_get_path('module', 'translatable') .'/translatable.css');
  $languages = translatable_available_locales();
  $style = variable_get('translatable_block_'. $block .'_style', 'flag-name');

  // When editing a node, fetch existent translations.
  if (!isset($tset) && arg(0) == 'node' && arg(2) == 'edit') {
    $tset = translatable_findbyid('node', arg(1));
    if ($tset) {
      $nodes = translatable_find('node', "tnid = ". $tset['tnid']);
    }
  }

  foreach ($languages as $lang => $langname) {
    $langname = translatable_t($langname, 0, $lang);
    // Skip active language if enabled.
    if (variable_get('translatable_block_'. $block .'_hidecurrentlocale', FALSE) && $lang == translatable_get_locale()) {
      continue;
    }
    switch ($style) {
      case 'flag':
        $title = theme('translatable_flag', $lang, $langname, $block);
        break;
        
      case 'name':
        $title = $langname;
        break;
        
      case 'flag-name':
        $title = theme('translatable_flag', $lang, $langname, $block) . $langname;
        break;
    }
    $attribs = array('title' => $langname, 'class' => 'language');
    $absolute = FALSE;

    // Determine redirect destination for links in translation language block
    // or both blocks, if user does not prefer separate languages.
    // @todo Only for enabled content types!
    if (arg(0) == 'node' && ($block == 'translation' || !$user->switch_adminlocale)) {
      if (arg(1) == 'add') {
        // When creating a new node, we simply switch the language and reload.
        $destination = $_GET['q'];
      }
      else if (isset($nodes)) {
        // When editing a node, we need to search all translations for the
        // current parent id to build corresponding links.
        $node = FALSE;
        foreach ($nodes as $tnode) {
          if ($tnode['language'] == $lang) {
            $node = $tnode;
            break;
          }
        }
        if ($node) {
          // A translation exists; redirect to regular node edit page.
          $destination = 'node/'. $node['nid'] .'/edit';
        }
        else {
          // No translation exists; redirect to our callback.
          $destination = 'node/'. $tset['tnid'] .'/edit/translation/'. $lang;
        }
      }
    }

    switch ($block) {
      case 'ui':
        if (!isset($destination)) {
          $destination = translatable_get_destination(translatable_get_defaultcontentlocale(FALSE, $lang));
          if ($destination == '') {
            $destination = variable_get('site_frontpage', 'node');
          }
          if (strpos($destination, 'destination=') !== FALSE) {
            $destination = $_GET['q'];
          }
        }
        if (variable_get('translatable_switch_byhostname', FALSE)) {
          $absolute = TRUE;
          $host = variable_get('translatable_switch_hostname_'. $lang, $lang .'.'. translatable_get_domain());
          $url   = 'http://'. $host . $GLOBALS['base_path'];
          if (variable_get('clean_url', 0)) {
            $url .= $destination;
            $query = NULL;
          }
          else {
            $query = 'q='. $destination;
          }
        }
        else {
          $url   = 'switchuilocale/'. $lang;
          $query = 'destination='. drupal_urlencode($destination);
        }
        $attribs['class'] .= ($lang == translatable_get_locale() ? ' active' : '');
        break;
        
      case 'translation':
        if (isset($destination)) {
          $attribs['class'] .= ($lang == $form['language']['#default_value'] || $lang == translatable_get_adminlocale() ? ' active' : '');
        }
        else {
          // Simple translation form.
          $destination = $_GET['q'];
          $attribs['class'] .= ($lang == translatable_get_adminlocale() ? ' active' : '');
        }
        $message = t('Please wait...');
        $attribs['onclick'] = "$('#". str_replace('_', '-', $form['#id']) ."').block('$message');";
        $url   = 'switchadminlocale/'. $lang;
        $query = 'destination='. drupal_urlencode($destination);
        break;
    }
    $links[$lang] = l($title, $url, $attribs, $query, NULL, $absolute, TRUE);
  }
  return $links;
}

/**
 * Create the HTML for a language flag.
 * 
 * @param string $locale
 *   The ISO locale of the flag.
 * @param string $language
 *   The language name, used for alt and title.
 * @param string $block
 *   The name of the block in which the flag is displayed.
 *
 * @return
 *   HTML for a flag in the given language.
 */
function theme_translatable_flag($locale, $language, $block = 'ui') {
  $path = variable_get('translatable_block_'. $block .'_flagspath', drupal_get_path('module', 'translatable') .'/flags/*.png');
  $src = str_replace('*', $locale, $path);
  $attribs = array('class' => 'translatable-flag');
  return theme('image', $src, $language, $language, $attribs);
}

/**
 * Implementation of hook_user().
 */
function translatable_user($op, &$edit, &$account, $category = NULL) {
  global $user;
  
  switch ($op) {
    case 'after_update':
    case 'login':
      // Switch UI locale after updating the user account or logging in.
      $languages = translatable_available_locales();
      if (isset($languages[$account->language]) && $user->uid == $account->uid) {
        translatable_set_locale($account->language);
      }
      break;

    case 'load':
      // Apply default setting for switch_adminlocale if not defined.
      if (empty($account->switch_adminlocale)) {
        $account->switch_adminlocale = 0;
      }
      break;

    case 'form':
      $languages = translatable_available_locales();
      if (count($languages) < 2 || !user_access('access translatable') || $category != 'account') {
        return;
      }
      $form['locale']['switch_adminlocale'] = array(
        '#type' => 'radios',
        '#title' => t('Translation language'),
        '#options' => array(t('Always equivalent to interface language'), t('Separate translation language')),
        '#default_value' => $account->switch_adminlocale,
        '#weight' => 1,
        '#description' => t('Please choose, whether the translation language should always match the interface language or if it should be kept separated, when switching one of both languages.'),
      );
      return $form;
  }
}

/**
 * Switch the locale of an user, if enabled.
 *
 * @param $type
 *   Which locale to switch; 'locale' or 'adminlocale'.
 * @param $locale
 *   A language to switch to; already validated.
 */
function translatable_user_locale($type, $locale) {
  global $user;
  
  // Only switch if the user does not prefer separate locales.
  if (!$user->switch_adminlocale) {
    $function = 'translatable_set_'. $type;
    $function($locale, FALSE);
  }
}

