<?php
// $Id: translatable.page-translations.inc,v 1.1 2008/02/17 05:22:40 sun Exp $

/**
 * Callback for translation management page.
 */
function translatable_page_translations($op = '', $args = array()) {
  $output = '';
  $op = isset($_POST['op']) ? $_POST['op'] : $op;

  switch ($op) {
    case 'select':
      $node = node_load(arg(1));
      $output .= translatable_node_overview($node);
      $output .= translatable_node_select($node, $args[0] ? $args[0] : translatable_get_adminlocale());
      break;
    
    case t('Assign'):
      $node = node_load(arg(1));
      $output .= translatable_node_select($node, $args[0] ? $args[0] : translatable_get_adminlocale());
      break;
    
    case 'remove':
      if ($node = node_load($args[0])) {
        $translatable_node = translatable_findbyid('node', $node->nid);
        if ($translatable_node['tnid'] != $node->nid) {
          translatable_save('node', array('nid' => $node->nid, 'tnid' => $node->nid, 'language' => $node->language));
          drupal_set_message('The content has been removed from the translation set.');
        }
        else {
          drupal_set_message('The source content cannot be removed from the translation set.', 'error');
        }
        drupal_goto('node/'. arg(1) .'/translations');
      }

    default:
      if (!isset($node)) {
        $node = node_load(arg(1));
      }
      $output .= translatable_node_overview($node);
  }
  return $output;
}

/**
 * Hook for the overview of a node.
 * Implements the translations tab interface.
 */
function translatable_node_overview($node) {
  drupal_set_title(t('Translations of %content', array('%content' => $node->title)));
  
  $languages = translatable_available_locales();
  $header = array(t('Language'), t('Title'), t('Status'), t('Operations'));
  $rows = array();
  $status_text = array(0 => t('Unpublished'), 1 => t('Published'), -1 => t('Not translated'));
  foreach ($languages as $lang => $langname) {
    $options = array();
    $translatable_node = translatable_find('node', "tnid='$node->tnid' AND language='$lang'", FALSE);
    if ($translatable_node['nid']) {
      $trnode = db_fetch_object(db_query('SELECT n.nid, n.title, loc.language, n.status, loc.any FROM {node} n INNER JOIN {node_translatable} loc ON n.nid = loc.nid AND n.nid = %d', $translatable_node['nid']));
      $title = l($trnode->title, 'node/'. $trnode->nid, NULL, 'locale='. $trnode->language);
      if ($trnode->nid == $node->tnid) {
        $langname = '<strong>'. $langname .'</strong> '. t('(source)');
      }
      if ($trnode->any) {
        $langname .= ' '. t('(any language)');
      }
      $status = $status_text[$trnode->status];
      $options[] = l(t('Edit'), "node/$trnode->nid/edit");
      if ($trnode->nid != $node->tnid) {
        $options[] = l(t('Remove'), "node/$node->nid/translations/remove/$trnode->nid");
      }
    }
    else {
      $title = t('n/a');
      $status = $status_text[-1];
      $options[] = l(t('Create'), "node/$node->nid/edit/translation/$lang");
      $options[] = l(t('Select'), "node/$node->nid/translations/select/$lang");
    }
    $rows[] = array($langname, $title, $status, implode(" | ", $options));
  }
  $output .= theme('table', $header, $rows);
  return $output;
}

function translatable_node_select($node, $locale) {
  // Sanity check.
  if (!translatable_validate_locale($locale)) {
    return;
  }
  $form['node'] = array('#type' => 'value', '#value' => $node);
  $languages = translatable_available_locales();
  
  // Only select nodes that do not belong to another translation set.
  $selectable = translatable_find('node', "", TRUE, TRUE, array('group by' => 'tnid', 'having' => array('COUNT(tnid) = 1', "language = '$locale'")));
  if (!empty($selectable)) {
    $result = pager_query(db_prefix_tables("SELECT n.nid, n.title FROM {node} n WHERE n.nid IN (%s) AND n.type = '%s' ORDER BY n.title"), 40, 0, NULL, implode(',', array_keys($selectable)), $node->type);
    
    while ($trnode = db_fetch_object($result)) {
      $list[$trnode->nid] = l($trnode->title, "node/$trnode->nid");
    }
  }
  if (!empty($list)) {
    $output = drupal_get_form('translatable_node_select_form', $node, $locale, $list);
  }
  else {
    $output = t("No unassigned contents available in %language.", array('%language' => $languages[$locale]));
  }
  return $output;
}

function translatable_node_select_form($node, $locale, $list) {
  $languages = translatable_available_locales();
  $translations = translatable_node_get_translations($node->nid);
  $form['#submit'] = array('translatable_node_select_form_submit' => array());
  $form['tnid'] = array(
    '#type' => 'hidden',
    '#value' => $node->tnid,
  );
  $form['language'] = array(
    '#type' => 'hidden',
    '#value' => $locale,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => "node/$node->nid/translations",
  );
  $form['nodes']['nid'] = array(
    '#type' => 'radios',
    '#title' => t('Select %language translation', array('%language' => $languages[$locale])),
    '#default_value' => isset($translations[$locale]) ? $translations[$locale]->nid : '',
    '#options' => $list,
    '#description' => t('<strong>Note:</strong> This list only contains contents that do not belong to another translation set.  If a content from another translation set should be assigned to this translation set, that content has to be removed from the existing set first.'),
  );
  $form['pager'] = array('#value' => theme('pager'));
  $form['submit'] = array('#type' => 'submit', '#value' => t('Assign'));
  return $form;
}

function translatable_node_select_form_submit($form_id, $form_values) {
  if ($form_values['tnid'] && $form_values['nid']) {
    translatable_save('node', array(
      'nid' => $form_values['nid'],
      'tnid' => $form_values['tnid'],
      'language' => $form_values['language'],
    ));
    drupal_set_message(t('The translation has been assigned to this content.'));
    drupal_goto($form_values['redirect']);
  }
}

/**
 * Gets the translations for the current node.
 *
 * @param $nid
 *   The id of the node.
 * @param $getitself
 *   Includes also the node in the list.
 */
function translatable_node_get_translations($nid, $getitself = FALSE) {
  $translatable_node = translatable_findbyid('node', $nid);
  $tnid = $translatable_node['tnid'];
  
  $query = 'SELECT n.nid, n.title, n.status, loc.language FROM {node} n INNER JOIN {node_translatable} loc ON n.nid = loc.nid AND loc.tnid = %d';
  if (!$getitself) {
    $query .= ' AND loc.nid <> %d';
  }
  $result = db_query($query, $tnid, $nid);
  
  $nodes = array();
  while ($node = db_fetch_object($result)) {
    $nodes[$node->language] = $node;
  }
  return $nodes;
}


